/*
 * $Id:  $
 *
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.security;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.SSLSocketFactory;

/**
 * This implementation of SecurityContext provides functionality equivalent to
 * how official vijava's ServiceInstance works when ignoreCert is true. Essentially,
 * all destination hostnames and SSL certificates are trusted, which is not ideal
 * in a production enviroment. However, this functionality is provided to ease the
 * transition over to a more secure implementation (either ConfigurableSecurityContext
 * or CustomSSLSocketFactorySecurityContext.)
 *
 * TODO Move to a common security library
 */
public class TrustAllSecurityContext implements SecurityContext {
    private final ConfigurableSecurityContext delegate = new ConfigurableSecurityContext()
            .trustManager(new TrustAllManager());

    @Override
    public HostnameVerifier getHostnameVerifier() {
        return delegate.getHostnameVerifier();
    }

    @Override
    public SSLSocketFactory getSslSocketFactory() {
        return delegate.getSslSocketFactory();
    }
}
